@@ -9,10 +9,11 @@ demo applications based on the gui_app and the related GUI framework implementat
 from abc import abstractmethod, ABC
 from typing import Any, Dict, List, Optional, Sequence
 
-from ae.gui_app import id_of_flow, flow_action, flow_key, replace_flow_action        # type: ignore
+from ae.core import DEBUG_LEVEL_VERBOSE                                             # type: ignore
+from ae.gui_app import id_of_flow, flow_action, flow_key, replace_flow_action       # type: ignore
 
 
-__version__ = '0.0.3'
+__version__ = '0.0.4'
 
 
 FLOW_PATH_ROOT_ID = '<<<root>>>'     #: needed for flow path jumper
@@ -101,45 +102,52 @@ class LiszDataMixin(ABC):
         if node_only:
             nid.pop('node')
         else:
+            assert nid in self.current_node_items, f"DEL item data: {nid} not in {self.current_node_items}"
             self.current_node_items.remove(nid)
-            self.change_app_state('flow_id', id_of_flow('', ''), send_event=False)
+            self.change_flow(id_of_flow('', ''))
 
         self.play_sound('deleted')
 
-    def edit_validate(self, old_idx: int, new_id: str, want_node: bool) -> str:
+    def edit_validate(self, old_idx: int,
+                      new_id: Optional[str] = None, want_node: Optional[bool] = None, toggle_sel: bool = False) -> str:
         """ validate the user changes after adding a new item or editing an existing item.
 
         :param old_idx:         index in the current node of the edited item or -1 if a new item got edited.
         :param new_id:          new/edited id string.
-        :param want_node:       True if the new/edited item will have a sub-node.
+        :param want_node:       True if the new/edited item will have a sub-node, False if not.
+        :param toggle_sel:      True if the selection state of the item has to be toggled.
         :return:                empty string on successful validation, else error string or
                                 `'request_delete_confirmation_for_item'` if the user has to confirm the deletion
                                 after the user wiped the item id string or
                                 `'request_delete_confirmation_for_node'` if the user has to confirm the
                                 removal of the sub-node.
         """
-        if not check_item_id(new_id):
-            return f"invalid item name/id {new_id} containing '{FLOW_PATH_TEXT_SEP}' or is equal '{FLOW_PATH_ROOT_ID}'"
-
         add_item = old_idx == -1
-        if not new_id:
-            # on empty id cancel edit if add_item, else request confirmation from user for item del
-            return "" if add_item else 'request_delete_confirmation_for_item'
+        if new_id is None:
+            new_idx = old_idx
+        else:
+            if not new_id:
+                # on empty id cancel edit if add_item, else request confirmation from user for item del
+                return "" if add_item else 'request_delete_confirmation_for_item'
+            if not check_item_id(new_id):
+                return f"invalid item id {new_id} containing '{FLOW_PATH_TEXT_SEP}' or is equal '{FLOW_PATH_ROOT_ID}'"
 
-        new_idx = self.find_item_index(new_id)
-        if new_idx != -1 and (add_item or new_idx != old_idx):
-            return f"item id {new_id} exists already"
+            new_idx = self.find_item_index(new_id)
+            if new_idx != -1 and (add_item or new_idx != old_idx):
+                return f"item id '{new_id}' exists already"
 
         if add_item:                        # add new item
             nid = dict(id=new_id)
             if want_node:
                 nid['node'] = list()        # type: ignore   # mypy not supports recursive types see issue #731
             self.current_node_items.insert(0, nid)
+            new_idx = 0
 
         else:                               # edit item
             nid = self.current_node_items[old_idx]
-            nid['id'] = new_id
-            if want_node != ('node' in nid):    # NOTE: != operator has lower priority than in operator
+            if new_id:
+                nid['id'] = new_id
+            if want_node is not None and want_node != ('node' in nid):  # NOTE: != has lower priority than in operator
                 if want_node:
                     nid['node'] = list()        # type: ignore   # mypy not supports recursive types see issue #731
                 elif nid['node']:       # let user confirm node deletion of non-empty nid['node']
@@ -147,6 +155,9 @@ class LiszDataMixin(ABC):
                 else:
                     nid.pop('node')     # remove empty node
 
+        if toggle_sel:
+            self.toggle_item_sel(new_idx)
+
         self.play_sound('added' if add_item else 'edited')
 
         return ""
@@ -170,11 +181,11 @@ class LiszDataMixin(ABC):
 
         Used to display unique part of the key of the focused item/node.
 
-        :param flow_id:     flow id to get key to check from (pass the observed value to update GUI automatically,
-                            either self.app_state_flow_id or self.app_states['flow_id']).
-        :param landscape:   True if window has landscape shape (resulting in larger abbreviation). Pass the observed
-                            attribute, mostly situated in the framework_win (e.g. self.framework_win.landscape).
-        :return:            display text containing flow key.
+        :param flow_id:         flow id to get key to check from (pass the observed value to update GUI automatically,
+                                either self.app_state_flow_id or self.app_states['flow_id']).
+        :param landscape:       True if window has landscape shape (resulting in larger abbreviation). Pass the observed
+                                attribute, mostly situated in the framework_win (e.g. self.framework_win.landscape).
+        :return:                display text containing flow key.
         """
         if flow_action(flow_id) == 'focus':
             key = flow_key(flow_id)
@@ -186,15 +197,15 @@ class LiszDataMixin(ABC):
                     while item_id.startswith(key[:id_len]) and id_len < key_len:
                         id_len += 1
             return f" ->{key[:id_len]}"
-        return f".{flow_id}" if self.debug_level else ""
+        return f".{flow_id}" if self.debug_level >= DEBUG_LEVEL_VERBOSE else ""
 
     def flow_path_from_text(self, text: str) -> List[str]:
         """ restore the full complete flow path from the shortened flow keys generated by :meth:`.flow_path_text`.
 
-        :param text:    flow path text - like returned by :meth:`~LiszDataMixin.flow_path_text`.
-        :return:        flow path list.
+        :param text:            flow path text - like returned by :meth:`~LiszDataMixin.flow_path_text`.
+        :return:                flow path list.
         """
-        path_nid = None     # suppress Pycharm PyUnboundLocalVariable inspection warning
+        path_nid = None         # suppress Pycharm PyUnboundLocalVariable inspection warning
         flow_path = list()
         if text != FLOW_PATH_ROOT_ID:
             node = self.root_node
@@ -208,40 +219,11 @@ class LiszDataMixin(ABC):
                 node = path_nid['node']         # type: ignore   # mypy not supports recursive types see issue #731
         return flow_path
 
-    def flow_path_text(self, flow_path: List[str], min_len: int = 3) -> str:
-        """ generate shortened display text from the passed flow path.
-
-        :param min_len:
-        :param flow_path:
-        :return:
-        """
-        path_nid = None     # suppress Pycharm PyUnboundLocalVariable inspection warning
-        shortened_ids = list()
-        node = self.root_node
-        for flow_id in flow_path:
-            if flow_action(flow_id) != 'enter':
-                break
-            id_len = min_len
-            node_id = flow_key(flow_id)
-            sub_id_len = len(node_id)
-            for nid in node:
-                if nid['id'] == node_id:
-                    assert 'node' in nid
-                    path_nid = nid
-                elif 'node' in nid:
-                    while nid['id'].startswith(node_id[:id_len]) and id_len < sub_id_len:
-                        id_len += 1
-
-            shortened_ids.append(node_id[:id_len])
-            node = path_nid['node']         # type: ignore   # mypy not supports recursive types see issue #731
-
-        return FLOW_PATH_TEXT_SEP.join(shortened_ids)
-
     def flow_path_node(self, flow_path: List[str] = None) -> LiszNode:
         """ determine the current node specified by the passed flow path.
 
-        :param flow_path:
-        :return:
+        :param flow_path:       flow path list.
+        :return:                current node list.
         """
         if flow_path is None:
             flow_path = self.flow_path
@@ -264,7 +246,7 @@ class LiszDataMixin(ABC):
                     append_nodes(nid['node'])
                     deeper_flow_path.pop()
 
-        paths = [FLOW_PATH_ROOT_ID] if self.flow_path else list()
+        paths = [FLOW_PATH_ROOT_ID] if self.flow_path_action(0) == 'enter' else list()
         deep = 0
         while deep < len(self.flow_path) and self.flow_path_action(deep) == 'enter':
             deep += 1
@@ -275,10 +257,40 @@ class LiszDataMixin(ABC):
 
         return paths
 
+    def flow_path_text(self, flow_path: List[str], min_len: int = 3) -> str:
+        """ generate shortened display text from the passed flow path.
+
+        :param flow_path:       flow path list.
+        :param min_len:         minimum length of node ids (flow id keys).
+        :return:
+        """
+        path_nid = None         # suppress Pycharm PyUnboundLocalVariable inspection warning
+        shortened_ids = list()
+        node = self.root_node
+        for flow_id in flow_path:
+            if flow_action(flow_id) != 'enter':
+                break
+            id_len = min_len
+            node_id = flow_key(flow_id)
+            sub_id_len = len(node_id)
+            for nid in node:
+                if nid['id'] == node_id:
+                    assert 'node' in nid
+                    path_nid = nid
+                elif 'node' in nid:
+                    while nid['id'].startswith(node_id[:id_len]) and id_len < sub_id_len:
+                        id_len += 1
+
+            shortened_ids.append(node_id[:id_len])
+            if path_nid:                        # prevent error in quick jump to root
+                node = path_nid['node']         # type: ignore   # mypy not supports recursive types see issue #731
+
+        return FLOW_PATH_TEXT_SEP.join(shortened_ids)
+
     def focus_neighbour_item(self, delta: int):
         """ move flow id to previous/next displayed/filtered node item.
 
-        :param delta:       moving step (if greater 0 then forward, else backward).
+        :param delta:           moving step (if greater 0 then forward, else backward).
         """
         filtered_indexes = self.filtered_indexes
         if filtered_indexes:
@@ -349,9 +361,9 @@ class LiszDataMixin(ABC):
     def on_key_press(self, modifiers: str, key_code: str) -> bool:
         """  check key press event for to be handled and processed as command/action.
 
-        :param modifiers:   modifier keys string.
-        :param key_code:    key code string.
-        :return:            True if key event got processed/used, else False.
+        :param modifiers:       modifier keys string.
+        :param key_code:        key code string.
+        :return:                True if key event got processed/used, else False.
         """
         if modifiers or flow_action(self.flow_id) not in ('', 'focus'):
             return False
@@ -372,12 +384,11 @@ class LiszDataMixin(ABC):
             self.focus_neighbour_item(999999)
 
         # toggle selection of current item
-        elif key_code == ' ' and self.flow_id:    # key string 'space' is not in Window.command_keys
-            self.toggle_item_sel(self.find_item_index(flo_key))
-            self.refresh_node_widgets()
+        elif key_code == ' ' and flo_key:    # key string 'space' is not in Window.command_keys
+            self.change_flow(id_of_flow('save', 'item', flo_key), toggle_sel=True)
 
         # enter/leave flow in current list
-        elif key_code in ('enter', 'right') and self.flow_id and 'node' in self.item_by_id(flo_key):
+        elif key_code in ('enter', 'right') and flo_key and 'node' in self.item_by_id(flo_key):
             self.change_flow(id_of_flow('enter', 'item', flo_key))
         elif key_code in ('escape', 'left') and self.flow_path:
             self.change_flow(id_of_flow('leave', 'item'))
@@ -386,18 +397,21 @@ class LiszDataMixin(ABC):
         elif key_code in ('a', '+'):
             self.change_flow(id_of_flow('add', 'item'))
 
-        elif key_code == 'e' and self.flow_id:
+        elif key_code == 'e' and flo_key:
             self.change_flow(replace_flow_action(self.flow_id, 'edit'))  # popup_kwargs=dict(parent=self.framework_win))
 
-        elif key_code in ('-', 'del') and self.flow_id:
-            self.change_flow(id_of_flow('confirm', 'item_deletion'), item_id=flo_key)
+        elif key_code in ('-', 'del') and flo_key:
+            self.change_flow(id_of_flow('confirm', 'item_deletion', flo_key))
+
+        elif key_code == 'r':
+            self.refresh_all()
 
         else:
             return False    # pressed key not processable in the current flow/app-state
 
         return True         # key press processed
 
-    def on_node_enter(self, _flow_id: str, _event_kwargs: dict):
+    def on_item_enter(self, _flow_id: str, _event_kwargs: dict) -> bool:
         """ entering sub node from current node.
 
         :param _flow_id:        flow id.
@@ -405,8 +419,9 @@ class LiszDataMixin(ABC):
         :return:                True for to process/change flow id.
         """
         self.play_sound(id_of_flow('enter', 'item'))
+        return True
 
-    def on_node_leave(self, _flow_id: str, _event_kwargs: dict):
+    def on_item_leave(self, _flow_id: str, _event_kwargs: dict) -> bool:
         """ leaving sub node, setting current node to parent node.
 
         :param _flow_id:        flow id.
@@ -414,14 +429,15 @@ class LiszDataMixin(ABC):
         :return:                True for to process/change flow id.
         """
         self.play_sound(id_of_flow('leave', 'item'))
+        return True
 
-    def on_node_jump(self, flow_id: str, _event_kwargs: dict) -> bool:
+    def on_node_jump(self, flow_id: str, event_kwargs: dict) -> bool:
         """ FlowButton clicked event handler restoring flow path from the flow key.
 
         Used for to jump to node specified by the flow path text in the passed flow_id key.
 
         :param flow_id:         jump node flow with the flow key containing the flow path text.
-        :param _event_kwargs:   event arguments (not used here).
+        :param event_kwargs:    event arguments (not used here).
         :return:                True for to process/change flow.
         """
         flow_path = self.flow_path_from_text(flow_key(flow_id))
@@ -430,42 +446,43 @@ class LiszDataMixin(ABC):
         # an because flow_path_from_text() is overwriting the open popup action in self.flow_path
         # we have to re-add the latest flow id entry from the current/old flow path that opened the jumper
         # here (for it can be removed by FlowPopup closed event handler when the jumper popup closes).
-        open_jumper_flow_id = id_of_flow('open', 'flow_path_jumper')
-        assert open_jumper_flow_id == self.flow_path[-1]
+        # open_jumper_flow_id = id_of_flow('open', 'flow_path_jumper')
+        # assert open_jumper_flow_id == self.flow_path[-1]
         flow_path.append(self.flow_path[-1])
-        self.change_app_state('flow_path', flow_path)
-        self.change_app_state('flow_id', id_of_flow('', ''))
 
+        self.change_app_state('flow_path', flow_path, send_event=False)
+        event_kwargs['flow_id'] = id_of_flow('', '')
         self.play_sound(id_of_flow('enter', 'item'))
-        self.refresh_all()
+
         return True
 
     def refresh_all(self):
         """ refresh currently displayed items after changing current node. """
         assert not self._refreshing_data
         self._refreshing_data = True
-        if self.debug_level:
-            self.play_sound('debug_draw')
+        try:
+            if self.debug_level:
+                self.play_sound('debug_draw')
 
-        self.refresh_current_node_items_from_flow_path()
+            self.refresh_current_node_items_from_flow_path()
 
-        # save last actual flow id (because refreshed/redrawn widget observers could change flow id via focus)
-        flow_id = self.flow_id
+            # save last actual flow id (because refreshed/redrawn widget observers could change flow id via focus)
+            flow_id = self.flow_id
 
-        self.refresh_node_widgets()
+            self.refresh_node_widgets()
 
-        redraw = False
-        if flow_action(flow_id) == 'focus' and self.find_item_index(flow_key(flow_id)) == -1:
-            flow_id = id_of_flow('', '')    # reset flow id because last focused item got filtered/deleted by user
-            redraw = True
+            if flow_action(flow_id) == 'focus':
+                item_idx = self.find_item_index(flow_key(flow_id))
+                if item_idx not in self.filtered_indexes:
+                    flow_id = id_of_flow('', '')  # reset flow id because last focused item got filtered/deleted by user
 
-        self.change_app_state('flow_id', flow_id, send_event=redraw)    # correct flow or restore silently
+            self.change_app_state('flow_id', flow_id, send_event=False)     # correct flow or restore silently
 
-        if flow_action(flow_id) == 'focus':
-            self.call_method('on_flow_widget_focused')                  # restore focus
-
-        assert self._refreshing_data
-        self._refreshing_data = False
+            if flow_action(flow_id) == 'focus':
+                self.call_method('on_flow_widget_focused')                  # restore focus
+        finally:
+            assert self._refreshing_data
+            self._refreshing_data = False
 
     def refresh_current_node_items_from_flow_path(self):
         """ refresh current node including the depending display node. """
